@@ -70,6 +70,7 @@
 #include <KDirModel>
 #include <KIO/CopyJob>
 #include <KIO/Job>
+#include <KIO/PreviewJob>
 #include <KProtocolInfo>
 #include <KRun>
 
@@ -513,7 +514,7 @@ void FolderModel::setViewAdapter(QObject* adapter)
         if (m_viewAdapter && !m_previewGenerator) {
             m_previewGenerator = new KFilePreviewGenerator(abstractViewAdapter, this);
             m_previewGenerator->setPreviewShown(m_previews);
-            m_previewGenerator->setEnabledPlugins(m_previewPlugins);
+            m_previewGenerator->setEnabledPlugins(m_effectivePreviewPlugins);
         }
 
         emit viewAdapterChanged();
@@ -545,15 +546,23 @@ QStringList FolderModel::previewPlugins() const
 
 void FolderModel::setPreviewPlugins(const QStringList& previewPlugins)
 {
-    if (m_previewPlugins != previewPlugins) {
-        m_previewPlugins = previewPlugins;
+    QStringList effectivePlugins = previewPlugins;
+    if (effectivePlugins.isEmpty()) {
+        effectivePlugins = KIO::PreviewJob::defaultPlugins();
+    }
+
+    if (m_effectivePreviewPlugins != effectivePlugins) {
+        m_effectivePreviewPlugins = effectivePlugins;
 
         if (m_previewGenerator) {
             m_previewGenerator->setPreviewShown(false);
-            m_previewGenerator->setEnabledPlugins(m_previewPlugins);
+            m_previewGenerator->setEnabledPlugins(m_effectivePreviewPlugins);
             m_previewGenerator->setPreviewShown(true);
         }
+    }
 
+    if (m_previewPlugins != previewPlugins) {
+        m_previewPlugins = previewPlugins;
         emit previewPluginsChanged();
     }
 }
@@ -998,7 +1007,8 @@ void FolderModel::dragSelectedInternal(int x, int y)
 static bool isDropBetweenSharedViews(const QList<QUrl> &urls, const QUrl &folderUrl)
 {
     for (const auto &url : urls) {
-        if (folderUrl != url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash)) {
+        if (folderUrl.adjusted(QUrl::StripTrailingSlash)
+               != url.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash)) {
             return false;
         }
     }
@@ -1107,6 +1117,11 @@ void FolderModel::drop(QQuickItem *target, QObject* dropEvent, int row)
         return;
     }
 
+    // Catch drops from a Task Manager and convert to usable URL.
+    if (!mimeData->hasUrls() && mimeData->hasFormat(QStringLiteral("text/x-orgkdeplasmataskmanager_taskurl"))) {
+        QList<QUrl> urls = {QUrl(QString::fromUtf8(mimeData->data(QStringLiteral("text/x-orgkdeplasmataskmanager_taskurl"))))};
+        mimeData->setUrls(urls);
+    }
 
     if (m_usedByContainment && !m_screenMapper->sharedDesktops()) {
         if (isDropBetweenSharedViews(mimeData->urls(), dropTargetFolderUrl)) {
@@ -1242,6 +1257,8 @@ void FolderModel::selectionChanged(const QItemSelection &selected, const QItemSe
             delete m_dragImages.take(idx.row());
         }
     }
+
+    updateActions();
 }
 
 bool FolderModel::isBlank(int row) const
@@ -1267,13 +1284,7 @@ QVariant FolderModel::data(const QModelIndex& index, int role) const
     } else if (role == SelectedRole) {
         return m_selectionModel->isSelected(index);
     } else if (role == IsDirRole) {
-        const QUrl &url = data(index, UrlRole).toUrl();
-
-        if (m_isDirCache.contains(url)) {
-            return m_isDirCache[url];
-        } else {
-            return isDir(mapToSource(index), m_dirModel);
-        }
+        return isDir(mapToSource(index), m_dirModel);
     } else if (role == IsLinkRole) {
         const KFileItem item = itemForIndex(index);
         return item.isLink();
@@ -1326,21 +1337,36 @@ bool FolderModel::isDir(const QModelIndex &index, const KDirModel *dirModel) con
         return true;
     }
 
+    auto it = m_isDirCache.constFind(item.url());
+    if (it != m_isDirCache.constEnd()) {
+        return *it;
+    }
+
     if (m_parseDesktopFiles && item.isDesktopFile()) {
         // Check if the desktop file is a link to a directory
         KDesktopFile file(item.targetUrl().path());
 
-        if (file.hasLinkType()) {
-            const QUrl url(file.readUrl());
-
-            if (!m_isDirCache.contains(item.url()) && KProtocolInfo::protocolClass(url.scheme()) == QStringLiteral(":local")) {
-                KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
-                job->setProperty("org.kde.plasma.folder_url", item.url());
-                job->setSide(KIO::StatJob::SourceSide);
-                job->setDetails(0);
-                connect(job, &KJob::result, this, &FolderModel::statResult);
-            }
+        if (!file.hasLinkType()) {
+            return false;
         }
+
+        const QUrl url(file.readUrl());
+
+        // Check if we already have a running StatJob for this URL.
+        if (m_isDirJobs.contains(item.url())) {
+            return false;
+        }
+
+        if (KProtocolInfo::protocolClass(url.scheme()) != QStringLiteral(":local")) {
+            return false;
+        }
+
+        KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
+        job->setProperty("org.kde.plasma.folder_url", item.url());
+        job->setSide(KIO::StatJob::SourceSide);
+        job->setDetails(0);
+        connect(job, &KJob::result, this, &FolderModel::statResult);
+        m_isDirJobs.insert(item.url(), job);
     }
 
     return false;
@@ -1353,11 +1379,13 @@ void FolderModel::statResult(KJob *job)
     const QUrl &url = statJob->property("org.kde.plasma.folder_url").toUrl();
     const QModelIndex &idx = index(indexForUrl(url), 0);
 
-    if (idx.isValid()) {
+    if (idx.isValid() && statJob->error() == KJob::NoError) {
         m_isDirCache[url] = statJob->statResult().isDir();
 
         emit dataChanged(idx, idx, QVector<int>() << IsDirRole);
     }
+
+    m_isDirJobs.remove(url);
 }
 
 void FolderModel::evictFromIsDirCache(const KFileItemList& items)
@@ -1411,8 +1439,8 @@ bool FolderModel::lessThan(const QModelIndex &left, const QModelIndex &right) co
                 break;
             }
         case KDirModel::ModifiedTime: {
-                const QDateTime leftTime = leftItem.time(KFileItem::ModificationTime);
-                const QDateTime rightTime = rightItem.time(KFileItem::ModificationTime);
+                const long long leftTime = leftItem.entry().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1);
+                const long long rightTime = rightItem.entry().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1);
                 if (leftTime < rightTime)
                     result = -1;
                 else if (leftTime > rightTime)
@@ -1587,19 +1615,52 @@ QObject* FolderModel::newMenu() const
 
 void FolderModel::updateActions()
 {
+    const QModelIndexList indexes = m_selectionModel->selectedIndexes();
+
+    KFileItemList items;
+    QList<QUrl> urls;
+    bool hasRemoteFiles = false;
+    bool isTrashLink = false;
+    const bool isTrash = (resolvedUrl().scheme() == QLatin1String("trash"));
+
+    if (indexes.isEmpty()) {
+        items << rootItem();
+    } else {
+        items.reserve(indexes.count());
+        urls.reserve(indexes.count());
+        for (const QModelIndex &index : indexes) {
+            KFileItem item = itemForIndex(index);
+            if (!item.isNull()) {
+                hasRemoteFiles |= item.localPath().isEmpty();
+                items.append(item);
+                urls.append(item.url());
+            }
+        }
+    }
+
+    KFileItemListProperties itemProperties(items);
+    // Check if we're showing the menu for the trash link
+    if (items.count() == 1 && items.at(0).isDesktopFile()) {
+        KDesktopFile file(items.at(0).localPath());
+        if (file.hasLinkType() && file.readUrl() == QLatin1String("trash:/")) {
+            isTrashLink = true;
+        }
+    }
+
     if (m_newMenu) {
         m_newMenu->checkUpToDate();
         m_newMenu->setPopupFiles(m_dirModel->dirLister()->url());
         // we need to set here as well, when the menu is shown via AppletInterface::eventFilter
         m_menuPosition = QCursor::pos();
+
+        if (QAction *newMenuAction = m_actionCollection.action(QStringLiteral("newMenu"))) {
+            newMenuAction->setEnabled(itemProperties.supportsWriting());
+            newMenuAction->setVisible(!isTrash);
+        }
     }
 
-    const bool isTrash = (resolvedUrl().scheme() == QLatin1String("trash"));
-
-    QAction *emptyTrash = m_actionCollection.action(QStringLiteral("emptyTrash"));
-
-    if (emptyTrash) {
-        if (isTrash) {
+    if (QAction *emptyTrash = m_actionCollection.action(QStringLiteral("emptyTrash"))) {
+        if (isTrash || isTrashLink) {
             emptyTrash->setVisible(true);
             emptyTrash->setEnabled(!isTrashEmpty());
         } else {
@@ -1611,9 +1672,20 @@ void FolderModel::updateActions()
         restoreFromTrash->setVisible(isTrash);
     }
 
-    QAction *paste = m_actionCollection.action(QStringLiteral("paste"));
+    if (QAction *moveToTrash = m_actionCollection.action(QStringLiteral("trash"))) {
+        moveToTrash->setVisible(!hasRemoteFiles && itemProperties.supportsMoving());
+    }
 
-    if (paste) {
+    if (QAction *del = m_actionCollection.action(QStringLiteral("del"))) {
+        del->setVisible(itemProperties.supportsDeleting());
+    }
+
+    if (QAction *cut = m_actionCollection.action(QStringLiteral("cut"))) {
+        cut->setEnabled(itemProperties.supportsDeleting());
+        cut->setVisible(!isTrash);
+    }
+
+    if (QAction *paste = m_actionCollection.action(QStringLiteral("paste"))) {
         bool enable = false;
 
         const QString pasteText = KIO::pasteActionText(QApplication::clipboard()->mimeData(),
@@ -1627,25 +1699,30 @@ void FolderModel::updateActions()
             paste->setEnabled(false);
         }
 
-        QAction* pasteTo = m_actionCollection.action(QStringLiteral("pasteto"));
-
-        if (pasteTo) {
+        if (QAction *pasteTo = m_actionCollection.action(QStringLiteral("pasteto"))) {
+            pasteTo->setVisible(itemProperties.isDirectory() && itemProperties.supportsWriting());
             pasteTo->setEnabled(paste->isEnabled());
             pasteTo->setText(paste->text());
         }
     }
+
+    if (QAction *rename = m_actionCollection.action(QStringLiteral("rename"))) {
+        qDebug() << "RIENÄIM" << itemProperties.supportsMoving() << itemProperties.urlList();
+        rename->setEnabled(itemProperties.supportsMoving());
+        rename->setVisible(!isTrash);
+    }
 }
 
 void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifiers modifiers)
 {
-    QModelIndexList indexes = m_selectionModel->selectedIndexes();
-
     if (m_usedByContainment && !KAuthorized::authorize(QStringLiteral("action/kdesktop_rmb"))) {
         return;
     }
 
     updateActions();
 
+    const QModelIndexList indexes = m_selectionModel->selectedIndexes();
+
     QMenu *menu = new QMenu();
     if (!m_fileItemActions) {
         m_fileItemActions = new KFileItemActions(this);
@@ -1668,41 +1745,25 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
     } else {
         KFileItemList items;
         QList<QUrl> urls;
-        bool hasRemoteFiles = false;
-        bool isTrashLink = false;
 
         items.reserve(indexes.count());
         urls.reserve(indexes.count());
-        foreach (const QModelIndex &index, indexes) {
+        for (const QModelIndex &index : indexes) {
             KFileItem item = itemForIndex(index);
             if (!item.isNull()) {
-                hasRemoteFiles |= item.localPath().isEmpty();
                 items.append(item);
                 urls.append(item.url());
             }
         }
-        KFileItemListProperties itemProperties(items);
 
-        // Check if we're showing the menu for the trash link
-        if (items.count() == 1 && items.at(0).isDesktopFile()) {
-            KDesktopFile file(items.at(0).localPath());
-            if (file.hasLinkType() && file.readUrl() == QLatin1String("trash:/")) {
-                isTrashLink = true;
-            }
-        }
+        KFileItemListProperties itemProperties(items);
 
         // Start adding the actions:
         menu->addAction(m_actionCollection.action(QStringLiteral("open")));
         menu->addSeparator();
-        if (itemProperties.supportsDeleting()) {
-            menu->addAction(m_actionCollection.action(QStringLiteral("cut")));
-        }
+        menu->addAction(m_actionCollection.action(QStringLiteral("cut")));
         menu->addAction(m_actionCollection.action(QStringLiteral("copy")));
-
-        if (itemProperties.isDirectory() && itemProperties.supportsWriting()) {
-            menu->addAction(m_actionCollection.action(QStringLiteral("pasteto")));
-        }
-
+        menu->addAction(m_actionCollection.action(QStringLiteral("pasteto")));
         menu->addAction(m_actionCollection.action(QStringLiteral("rename")));
         menu->addAction(m_actionCollection.action(QStringLiteral("restoreFromTrash")));
 
@@ -1710,28 +1771,19 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
         KConfigGroup cg(globalConfig, "KDE");
         bool showDeleteCommand = cg.readEntry("ShowDeleteCommand", false);
 
-        // When we're showing the menu for the trash link, offer "Empty Trash" instead
-        // of the "Move to Trash" action.
-        if (isTrashLink) {
-            QAction *emptyTrashAction = m_actionCollection.action(QStringLiteral("emptyTrash"));
-            if (emptyTrashAction) {
-                // We explicitly force the action visible here, as it relies on the KFileItemList
-                // we collected above. In updateActions() we don't have it and since this is always
-                // called before we open the menu, it would correct visibility again when opening
-                // the context menu for other items later.
-                emptyTrashAction->setVisible(true);
-                emptyTrashAction->setEnabled(!isTrashEmpty());
-                menu->addAction(emptyTrashAction);
-            }
+        menu->addAction(m_actionCollection.action(QStringLiteral("emptyTrash")));
+        if (modifiers.testFlag(Qt::ShiftModifier)) {
+            showDeleteCommand = true;
         } else {
-            if (!modifiers.testFlag(Qt::ShiftModifier) && !hasRemoteFiles && itemProperties.supportsMoving()) {
-                menu->addAction(m_actionCollection.action(QStringLiteral("trash")));
-            } else {
-                showDeleteCommand = true;
+            if (QAction *trashAction = m_actionCollection.action(QStringLiteral("trash"))) {
+                menu->addAction(trashAction);
+                if (!trashAction->isVisible()) {
+                    showDeleteCommand = true;
+                }
             }
         }
 
-        if (showDeleteCommand && itemProperties.supportsDeleting()) {
+        if (showDeleteCommand) {
             menu->addAction(m_actionCollection.action(QStringLiteral("del")));
         }
 
@@ -1755,12 +1807,10 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
 
         // Properties
         if (KPropertiesDialog::canDisplay(items)) {
-            QAction *act = new QAction(menu);
-            act->setText(i18n("&Properties"));
+            QAction *act = new QAction(QIcon::fromTheme(QStringLiteral("document-properties")), i18n("&Properties"), menu);
             QObject::connect(act, &QAction::triggered, this, &FolderModel::openPropertiesDialog);
             menu->addAction(act);
         }
-
     }
 
     if (visualParent) {
@@ -1768,6 +1818,10 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
     } else {
         m_menuPosition = QCursor::pos();
     }
+    
+    
+    menu->setAttribute(Qt::WA_TranslucentBackground);
+    menu->winId(); //force surface creation before ensurePolish call in menu::Popup which happens before show
     menu->popup(m_menuPosition);
     connect(menu, &QMenu::aboutToHide, [menu]() { menu->deleteLater(); });
 }
@@ -1821,6 +1875,12 @@ void FolderModel::copy()
         return;
     }
 
+    if (QAction *action = m_actionCollection.action(QStringLiteral("copy"))) {
+        if (!action->isEnabled()) {
+            return;
+        }
+    }
+
     QMimeData *mimeData = QSortFilterProxyModel::mimeData(m_selectionModel->selectedIndexes());
     QApplication::clipboard()->setMimeData(mimeData);
 }
@@ -1831,6 +1891,12 @@ void FolderModel::cut()
         return;
     }
 
+    if (QAction *action = m_actionCollection.action(QStringLiteral("cut"))) {
+        if (!action->isEnabled()) {
+            return;
+        }
+    }
+
     QMimeData *mimeData = QSortFilterProxyModel::mimeData(m_selectionModel->selectedIndexes());
     KIO::setClipboardDataCut(mimeData, true);
     QApplication::clipboard()->setMimeData(mimeData);
@@ -1838,6 +1904,12 @@ void FolderModel::cut()
 
 void FolderModel::paste()
 {
+    if (QAction *action = m_actionCollection.action(QStringLiteral("paste"))) {
+        if (!action->isEnabled()) {
+            return;
+        }
+    }
+
     KIO::paste(QApplication::clipboard()->mimeData(), m_dirModel->dirLister()->url());
 }
 
@@ -1896,6 +1968,12 @@ void FolderModel::moveSelectedToTrash()
         return;
     }
 
+    if (QAction *action = m_actionCollection.action(QStringLiteral("trash"))) {
+        if (!action->isEnabled()) {
+            return;
+        }
+    }
+
     const QList<QUrl> urls = selectedUrls();
     KIO::JobUiDelegate uiDelegate;
 
@@ -1912,6 +1990,12 @@ void FolderModel::deleteSelected()
         return;
     }
 
+    if (QAction *action = m_actionCollection.action(QStringLiteral("del"))) {
+        if (!action->isEnabled()) {
+            return;
+        }
+    }
+
     const QList<QUrl> urls = selectedUrls();
     KIO::JobUiDelegate uiDelegate;
 
